@@ -1,9 +1,17 @@
 package VIM::Packager;
-
 use warnings;
 use strict;
+use Exporter::Lite;
+our @EXPORT = qw(say);
+sub say { print @_ , "\n" }
 
-# ABSTRACT: Vim Packager builder / installer
+=head1 NAME
+
+VIM::Packager
+
+=cut
+
+our $VERSION = 2009.32120 ;
 
 =head1 SYNOPSIS
 
@@ -11,6 +19,9 @@ use strict;
 
 =head1 DESCRIPTION
 
+=head1 FUNCTIONS
+
+=cut
 
 =head1 REPOSITORY 
 
