@@ -2,6 +2,7 @@ package VIM::Packager::MakeMaker;
 use warnings;
 use strict;
 
+use VIM::Packager;
 use VIM::Packager::MetaReader;
 use VIM::Packager::Utils qw(vim_rtp_home vim_inst_record_dir findbin);
 use DateTime::Format::DateParse;
@@ -9,14 +10,11 @@ use YAML;
 use File::Spec;
 use File::Path;
 use File::Find;
+use VIM::Packager::Record;
 
 our $VERSION = 0.0.1;
 my  $VERBOSE = 1;
 
-use constant {
-    LIBPATH  => 'vimlib',
-};
-
 =head1 SYNOPSIS
 
     $ vim-packager build 
@@ -24,8 +22,31 @@ use constant {
         # auto install dependency 
     $ make install
 
+=head1 Constants
+
+LIBPATH: F<vimlib/>
+
 =cut
 
+use constant {
+    LIBPATH  => 'vimlib',
+};
+
+=head1 Makefile Helper Functions
+
+=head2 multi_line Array:Lines
+
+=head2 add_macro  ArrayRef:Makefile Lines , String: Macro Name , String: Macro 
+
+=head2 new_section ArrayRef:Makefile Lines , String: Section Name , Array: Depended macro names
+
+=head2 add_st ArrayRef:Makefile Lines , String: Statement , Array: Statement Arguments
+
+Statement Arguments is for Statement string, which is in sprintf format.
+
+=head2 add_noop_st ArrayRef:Makefile Lines
+
+=cut
 
 sub multi_line {
     my @items = @_;
@@ -45,23 +66,19 @@ sub new_section {
 }
 
 sub add_st {
-    my $ref = shift;
-    my $st = shift;
-    push @{ $ref } , qq|\t\t| . $st;
-}
-
-sub meta {
-    my $self = shift;
-    $self->{meta} = shift if @_;
-    return $self->{meta};
+    my ($ref, $st , @args ) = @_;
+    push @{ $ref } , qq|\t\t| . sprintf($st , @args);
 }
 
 sub add_noop_st {
 	add_st $_[0] => q|$(NOECHO) $(NOOP)|;
 }
 
+=head1 Main Functions
 
+=head2 new
 
+=cut
 
 sub new { 
     my $class = shift;
@@ -129,7 +146,7 @@ sub new {
     add_st $main => q|$(NOECHO) $(TOUCH) MANIFEST.SKIP|;
 
     new_section $main => 'dist';
-    add_st $main => q|$(TAR) $(TARFLAGS) $(DISTNAME).tar.gz $(TO_INST_VIMS)|;
+    add_st $main => q|$(TAR) $(TARFLAGS) $(DISTNAME).tar.gz $(TO_INST_VIMS) $(META_FILE) $(README)|;
 	add_noop_st $main;
 
     new_section $main => 'help';
@@ -147,9 +164,8 @@ sub new {
                 . multi_line qw|$(PWD)/$(DISTNAME).tar.gz $(VIM_VERSION) $(VERSION) $(SCRIPT_ID)|;
 
     new_section $main => 'clean';
-    add_st $main      => multi_line q|$(RM)|, qw(pure_install install-deps);
     add_st $main      => q|$(MV) $(FIRST_MAKEFILE) $(MAKEFILE_OLD)|;
-
+    add_st $main      => q|$(RM) *.tar.gz|;
 
     $self->generate_makefile( [
             { meta   => \@meta_section },
@@ -158,9 +174,28 @@ sub new {
             { main   => $main } ] );
 }
 
+=head2 meta
+
+return current meta object.
+
+=cut
+
+sub meta {
+    my $self = shift;
+    $self->{meta} = shift if @_;
+    return $self->{meta};
+}
 
 
 
+
+
+=head2 section_all
+
+
+
+=cut
+
 sub section_all {
     my $self = shift;
     my $main = shift;
@@ -168,6 +203,12 @@ sub section_all {
 	add_noop_st $main;
 }
 
+=head2 section_install
+
+
+
+=cut
+
 sub section_install {
     my $self = shift;
     my $main = shift;
@@ -175,6 +216,12 @@ sub section_install {
 	add_noop_st $main;
 }
 
+=head2 section_pure_install
+
+
+
+=cut
+
 sub section_pure_install {
     my ($self,$main,$makefile) = @_;
 
@@ -192,17 +239,18 @@ sub section_pure_install {
         }
     }
     else {
-
         add_st $main =>
             q|$(NOECHO) $(FULLPERL) $(PERLFLAGS) -MVIM::Packager::Installer=install|
-            . q| -e 'install()' $(VIMS_TO_RUNT) |;
-
-        add_st $main =>
-            q|$(NOECHO) $(FULLPERL) $(PERLFLAGS) -MVIM::Packager::Installer=install|
-            . q| -e 'install()' $(BIN_TO_RUNT) |;
+            . q| -e 'install()' $(NAME) $(VIMS_TO_RUNT) $(BIN_TO_RUNT)|;
     }
 }
 
+=head2 section_deps
+
+
+
+=cut
+
 sub section_deps {
     my $self = shift;
     my $main = shift;
@@ -211,17 +259,21 @@ sub section_deps {
 
     if( $self->{cmd}->{pure} ) {
         print "You are making a pure makefile that doesn't depend on perl module.\n";
-        print "We are going to skip deps section.\n";
+        print "We are going to skip dependency section.\n";
         add_noop_st $main;
         return;
     }
 
+    my $requires = $self->check_dependency( $self->meta );
+    my %unsatisfied      = %{ $requires->{unsat} };
+    my %nonversion_unsat = %{ $requires->{nonversion_unsat} };
 
-    my %unsatisfied = $self->check_dependency( $self->meta );
-    my @pkgs_nonversion = grep { ref($unsatisfied{$_}) eq 'ARRAY' } sort keys %unsatisfied;
+    # grep out the non-version package requires from those unsatisfied infomation
+    # which is an array ref because it's a file list.
+    my @pkgs_nonversion = keys %nonversion_unsat;
     for my $pkgname ( @pkgs_nonversion ) {
         my @nonversion_params = map {  ( $_->{target} , $_->{from} ) } 
-            map { @{ $unsatisfied{ $_ } } } $pkgname ;
+            map { @{ $nonversion_unsat{ $_ } } } $pkgname ;
 
         add_st $main => multi_line q|$(NOECHO) $(FULLPERL) $(PERLFLAGS)|
                     . qq| -MVIM::Packager::Installer=install_deps_remote |
@@ -230,13 +282,31 @@ sub section_deps {
 
     }
 
-    my @pkgs_version = grep {  ref($unsatisfied{$_}) ne 'ARRAY' } sort keys %unsatisfied;
+    # XXX: grep git repo deps from upstream ... zzz
+    # packages with version specified.
+    my @pkgs_version = keys %unsatisfied;
     if( @pkgs_version > 0 ) {
+        my @pkgs_git_repo = grep { $unsatisfied{ $_ }->{git_repo} } @pkgs_version;
+        for my $pkg ( @pkgs_git_repo ) {
+            my $dep = $unsatisfied{ $pkg };
+            
+            my $v = $dep->{version};
+            add_st $main => q|$(NOECHO) $(FULLPERL) $(PERLFLAGS) -MVIM::Packager::Installer=install_deps_from_git |
+                . qq| -e 'install_deps_from_git()' @{[ $dep->{git_repo} ]} $v|;
+        }
+
+        my @pkgs_version = grep { ! $unsatisfied{ $_ }->{git_repo} } @pkgs_version;
+        
         add_st $main => q|$(NOECHO) $(FULLPERL) $(PERLFLAGS) -MVIM::Packager::Installer=install_deps  |
-                . qq| -e 'install_deps()' '@{[ join ",",@pkgs_version ]}' |;
+                . qq| -e 'install_deps()' '@{[ join ",",@pkgs_version ]}' |
+                        if @pkgs_version > 0;
     }
 }
 
+=head2 section_link
+
+
+=cut
 
 sub section_link {
     my ($self,$main , $filelist) = @_;
@@ -316,24 +386,30 @@ sub config_section {
     my $perl = find_perl();
     die "Can not found perl." unless $perl;
 
-    $configs{FULLPERL} ||= $perl;
-    $configs{NOECHO}   ||= '@';
-    $configs{TOUCH}    ||= 'touch';
-    $configs{ECHO}     ||= 'echo';
-    $configs{ECHO_N}   ||= 'echo -n';
-    $configs{RM_F}     ||= "rm -vf";
-    $configs{RM_RF}    ||= "rm -rf";
-    $configs{TEST_F}   ||= "test -f";
-    $configs{CP}       ||= "cp";
-    $configs{MV}       ||= "mv";
-    $configs{CHMOD}    ||= "chmod";
-    $configs{FALSE}    ||= 'false';
-    $configs{TRUE}     ||= 'true';
-    $configs{NOOP}     ||= '$(TRUE)';
-    $configs{LN_S}     ||= 'ln -sv';
+    $configs{FULLPERL}  ||= $perl;
+    $configs{NOECHO}    ||= '@';
+    $configs{TOUCH}     ||= 'touch';
+    $configs{ECHO}      ||= 'echo';
+    $configs{ECHO_N}    ||= 'echo -n';
+    $configs{RM_F}      ||= "rm -vf";
+    $configs{RM_RF}     ||= "rm -rf";
+    $configs{TEST_F}    ||= "test -f";
+    $configs{CP}        ||= "cp";
+    $configs{MV}        ||= "mv";
+    $configs{CHMOD}     ||= "chmod";
+    $configs{FALSE}     ||= 'false';
+    $configs{TRUE}      ||= 'true';
+    $configs{NOOP}      ||= '$(TRUE)';
+    $configs{LN_S}      ||= 'ln -sv';
     $configs{LN_SF}     ||= 'ln -svf';
-    $configs{PWD}      ||= '`pwd`';
-    $configs{CP}       ||= 'cp -v';
+    $configs{PWD}       ||= '`pwd`';
+    $configs{CP}        ||= 'cp -v';
+
+    $configs{README} ||= '';
+    $configs{README} .= ' README'     if -e 'README';
+    $configs{README} .= ' README.mkd' if -e 'README.mkd';
+
+    $configs{META_FILE} ||= VIM::Packager::MetaReader->find_meta_file();
 
     $configs{FIRST_MAKEFILE} ||= 'Makefile';
     $configs{MAKEFILE_OLD}   ||= 'Makefile.old';
@@ -411,35 +487,49 @@ sub check_dependency {
     my @pkg_records = $self->get_installed_pkgs($record_dir);
 
     my %unsatisfied = ();
+    my %nonversion_unsat = ();
+
     for my $dep ( @{ $meta->{dependency} } ) {
 
         if ( defined $dep->{version} ) {
-            my ( $prereq, $required_version, $version_op ) = @$dep{qw(name version op)};
-
-            my $installed_files;  # XXX: get installed files of prerequire plugins
+            my ( $prereq, $required_version, $version_op , $git_repo ) 
+                        = @$dep{qw(name version op git_repo)};
 
             # XXX: check if prerequire plugin is installed. 
-            #      try to get installed package record by vimana manager 
+            #      try to get installed package record 
             #      or just look into file and parse the version
-            my $pr_version = 0 ; $pr_version = parse_version( $installed_files ) if $installed_files;  
+            my $pr_version = undef ;
 
-            if( ! $installed_files ) {
+            my $found = VIM::Packager::Record->find( $prereq );
+            if( $found ) {
+                my $r = VIM::Packager::Record->read( $found );
+                $pr_version = $r->{meta}->{version};
+                print "Found Installed Package: $prereq \n";
+                print "Version: $pr_version\n";
+            }
+            else {
+                my $installed_files; # get installed files here
+                $pr_version = parse_version( $installed_files ) if( $installed_files );
+            }
+
+            if( ! $pr_version ) {
                 warn sprintf "Warning: prerequisite %s - %s not found.\n", 
-                $prereq, $required_version;
-
-                $unsatisfied{ $prereq } = 'not installed';
+                    $prereq, $required_version;
+                $unsatisfied{ $prereq } = $dep;
             }
             elsif ( eval "$pr_version $version_op $required_version"  ) {
                 warn sprintf "Warning: prerequisite %s - %s not found. We have %s.\n",
-                    $prereq, $required_version, ($pr_version || 'unknown version') ;
-                $unsatisfied{ $prereq } = $pr_version;
+                    $prereq, $required_version, $pr_version;
+
+                $unsatisfied{ $prereq } = $dep;
             }
+
         }
         else {
             # if we can not detect installed package version
             # here is the other way to install dependencies.
             my ( $prereq , $require_files ) = ( $dep->{name}  , $dep->{required_files} );
-            $unsatisfied{ $prereq } = $require_files; 
+            $nonversion_unsat{ $prereq } = $require_files; 
 
             # XXX: grep out ?
             for ( @$require_files ) {
@@ -459,7 +549,10 @@ sub check_dependency {
 
     }
 
-    return %unsatisfied;
+    return {
+        unsat => \%unsatisfied,
+        nonversion_unsat => \%nonversion_unsat,
+    };
 }
 
 
